from qt import *
##~ from photolibrary import LPhotoLibrary
from utils import *

import os, errno
import shutil


class LImportItemEvent(QCustomEvent):
	id = QEvent.User+1
	def __init__(self, key, dict, progress, total, albumPath=None):
		QCustomEvent.__init__(self, self.id)
		self.setData(dict)
		self.key = key
		self.dict = dict
		self.progress = progress
		self.total = total
		self.albumPath = albumPath
		
	def getKey(self): return self.key		
	def getDict(self): return self.dict		
	def getAlbumPath(self): return self.albumPath
	def getProgress(self): return self.progress
	def getTotal(self): return self.total
		
		
class LImportStatusEvent(QCustomEvent):
	id = QEvent.User+2
	def __init__(self, progress, total):
		QCustomEvent.__init__(self, self.id)
		self.progress = progress
		self.total = total
		
	def getProgress(self): return self.progress		
	def getTotal(self): return self.total


class LImportCompleteEvent(QCustomEvent):
	id = QEvent.User+3
	def __init__(self, source):
		QCustomEvent.__init__(self, self.id)
		self.source = source
		
	def getSource(self): return self.source
		

##~ class LImportExistsEvent(QCustomEvent):
##~ 	id = QEvent.User+3
##~ 	def __init__(self, filename, progress, total, albumPath=None):
##~ 		QCustomEvent.__init__(self, self.id)
##~ 		self.filename = filename
##~ 		self.albumPath = albumPath
##~ 		self.progress = progress
##~ 		self.total = total
##~ 	
##~ 	def getFilename(self):
##~ 		return self.filename
##~ 
##~ 	def getAlbumPath(self):
##~ 		return self.albumPath
##~ 
##~ 	def getProgress(self):
##~ 		return (self.progress, self.total)


class LImportThread(QThread):
	def __init__(self, target):
		QThread.__init__(self)
		self.target = target
		self._validExtensions = [".jpg",".jpeg"]
		
	def setImportObject(self, source):
		self.source = source
		
	def __filenameGenerator(self, path, filename):
		yield filename
		(a,b) = os.path.splitext(filename)
		i = 1
		while True:
			yield a + str(i) + b
			i=i+1
		
	def run(self):
		from photolibrary import LPhotoLibrary
		from utils import LUtils
		import time
		
		files = self.source.getFileList()
		print "import thread file count=", len(files)
		i = 0
		total = len(files)
		# send zero event to init progress bar
		QApplication.postEvent(self.target, LImportStatusEvent(i, total))
		for file in files:
			i = i + 1
			print file
			tmpPath = self.source.copyToFilesystem(file)
			filename = self.source.getFilename(file)
			albumPath = self.source.getInsertAlbumPath(file)
			#(path, filename) = os.path.split(str(file))
			
			#build dictionary of image propertiesAlbum 7
			mtime = os.path.getmtime(str(tmpPath))
			importDestPath = str(os.path.join(LPhotoLibrary.libraryPath(), LUtils.datePath(mtime)))
			LUtils.ensureDirectory(importDestPath)
			thumbPath = os.path.join(importDestPath, "Thumbs")
			LUtils.ensureDirectory(thumbPath)
			photo = {}
			photo['Date'] = mtime
			photo['ImportDate'] = int(time.time())
			photo['Caption'] = filename
			photo['Rating'] = 0
			photo['Comments'] = ""
			
			proposedPath = str(os.path.join(importDestPath, filename))
			backupPath = str(os.path.join(importDestPath,"originals",filename))
			key = filename
			
			#copy file into library
			for filename in self.__filenameGenerator(importDestPath, filename):
				print "checking file",filename
				proposedPath = str(os.path.join(importDestPath, filename))
				key = filename
				if not LUtils.fileExists(proposedPath):
					break
				
				# check if file is the same size
				if ( mtime == os.path.getmtime(proposedPath)
					# destination file already exists, but is not an exact copy of this image
					# check to see if this image has a backup copy that matches
					or 
					LUtils.fileExists(backupPath) and mtime == os.path.getmtime(backupPath)):
					# if the backup file matches, this image has already been imported but is modified.
					# we will treat this as a match (but it would be nice to treat a drag differently)
					print "found import file"
					self.source.cleanupFilesystem(tmpPath)
					key = None
					break
					
					
				print "done checking"
			
						
			if key:  # if key, then this is a new file, move into library
				self.source.copyToLibrary(tmpPath, proposedPath)
			
			photo['ImagePath'] = proposedPath
			photo['ThumbPath'] = str(os.path.join(thumbPath, filename))
				
			#create thumbnail
			(w,h) = LUtils.downsizeImage(proposedPath, photo['ThumbPath'])
			if h==0 or w==0:
				continue # no width or height, this image is corrupt
				
			photo['AspectRatio'] = w/float(h)
			photo['Width'] = w
			photo['Height'] = h
			print "postImport",key,filename
			QApplication.postEvent(self.target, LImportItemEvent(key, photo, i, total, albumPath))
			
		#photo dict is complete, send back to library
		print "importthread complete"
		QApplication.postEvent(self.target, LImportCompleteEvent(self.source))
